/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __NI_COMMON_H__
#define __NI_COMMON_H__

#include <functional>

typedef std::function<void (std::string)> afterCreate;

/**
 * @brief : structure which contains directory info
 */
typedef struct NiCommonOption {
	std::string runtimeDir; /**< .NETCore Runtime directory */
	std::string tizenFXDir; /**< TizenFX directory */
	std::string extraDirs; /**< ":" seperated directories which can be set by plugins */
} NiCommonOption;

/**
 * @brief initialize NICommon
 * @param[in] options to initialize path
 * @return 0 on success, otherwise a negative error value
 */
int initNICommon(NiCommonOption* option);

/**
 * @brief finalize NICommon
 * @details Before re-initialize NICommon, you have to finalize current NICommon first.
 */
void finalizeNICommon();

/**
 * @brief create native images (NI file) for Platform DLLs (.NETCore + TizenFX)
 */
void createNiPlatform(bool enableR2R);

/**
 * @brief create native image for specific dll with file path.
 * @param[i] dllPath file path to generate native image
 * @param[i] enableR2R enable ready-to-run mode
 * @return 0 on success, otherwise a negative error value
 */
int createNiDll(const std::string& dllPath, bool enableR2R);

/**
 * @brief create native images with files under specific directory.
 * @param[i] rootPaths directories whicn contains DLLs
 * @param[i] count number of rootPath
 * @param[i] enableR2R enable ready-to-run mode
 */
void createNiUnderDirs(const std::string rootPaths[], int count, bool enableR2R);

/**
 * @brief create native images for specific package. (All DLLs)
 * @param[i] pkgId package ID
 * @param[i] enableR2R enable ready-to-run mode
 * @return 0 on success, otherwise a negative error value
 */
int createNiUnderPkgRoot(const std::string& pkgId, bool enableR2R);

/**
 * @brief create native image for specific dll in the package.
 * @Details All dlls in the package are added for reference when create native image.
 * @param[i] pkgId package ID
 * @param[i] enableR2R enable ready-to-run mode
 * @return 0 on success, otherwise a negative error value
 */
int createNiDllUnderPkgRoot(const std::string& pkgId, const std::string& dllPath, bool enableR2R);

/**
 * @brief remove native images (NI file) for Platform DLLs (.NETCore + TizenFX)
 */
void removeNiPlatform();

/**
 * @brief remove native images under specific directory.
 * @param[i] rootPaths directories whicn contains native image
 * @param[i] count number of rootPath
 */
void removeNiUnderDirs(const std::string rootPaths[], int count);

/**
 * @brief remove native images for specific package.
 * @param[i] pkgId package ID
 * @return 0 on success, otherwise a negative error value
 */
int removeNiUnderPkgRoot(const std::string& pkgId);

/**
 * @brief regenerate native image for all installed application
 */
int regenerateAppNI(bool enableR2R = true);

#endif /* __NI_COMMON_H__ */
